!c Description:
!c   ˰¾ηиη
!c
!c Current Code Owner: 
!c   sugiyama@gfd-dennou.org
!c
!c Histry: 
!c   Version    Date          Comment
!c   -------    ----------    --------
!c   1.0        2003-02-05    
!c
!c Copyright (C) SUGIYAMA Ko-ichiro, 2003, All rights reserved

module interface_def_svap_cef
  interface

     subroutine def_svap_cef(property, propertydir)
       use prm_type, only: type_property
       character(*), intent(in)                  :: propertydir
       type(type_property), intent(inout)        :: property(:)
     end subroutine def_svap_cef
     
  end interface
end module interface_def_svap_cef



subroutine def_svap_cef(property, propertydir)
  use prm_phys, only: pres_ref
  use prm_type, only: type_property
  use interface_netcdf_open 
  use interface_netcdf_get
  use interface_netcdf_close
  use interface_svap_amp
  use interface_svap_antoine
  use interface_gibbs_energy
  use interface_entropy

  implicit none
  character(*), intent(in)                  :: propertydir
  type(type_property), intent(inout)        :: property(:)
  
  real(8)                 :: temp
  real(8)                 :: svap
  real(8)                 :: dt
  real(8)                 :: entr0_dt
  real(8), allocatable    :: rvar(:) 
  real(8), allocatable    :: mol(:) 
  real(8), allocatable    :: entr0(:) 
  real(8), allocatable    :: entr0_pre(:) 
  real(8), allocatable    :: mu0(:) 
  real(8), parameter      :: pres_low = 1.0d2
  real(8), parameter      :: pres_high = 1.0d5
  real(8), parameter      :: temp_low = 5.0d1
  real(8), parameter      :: temp_high = 1.0d3
  integer                 :: ncid
  integer                 :: N, s, i
  integer, parameter      :: num = 10000
  character(80)           :: name
  character(80)           :: file
  

  !
  N = size(property, 1)
  allocate(mol(N))
  mol = 1.0d0   !ߡ
  property%svap_cef%low%temp = temp_low * 1.0d-1       
  property%svap_cef%low%A = 0.0d0
  property%svap_cef%low%B = 0.0d0
  property%svap_cef%low%C = 0.0d0
  property%svap_cef%high%temp = temp_high * 1.0d1
  property%svap_cef%high%A = 0.0d0
  property%svap_cef%high%B = 0.0d0
  property%svap_cef%high%C = 0.0d0
  property%svap_cef%A = 0.0d0
  property%svap_cef%B = 0.0d0
  property%svap_cef%C = 0.0d0
  property%svap_cef%D = 0.0d0
  property%svap_cef%E = 0.0d0


  calc_svap: do s = 1, N
     !׻ˡ 3, 4 ʳϽλ
     if (property(s)%method /= 3 .AND. property(s)%method /= 4) cycle calc_svap
     
     !ե̾
     file = trim(PropertyDir) // trim(property(s)%name) // '.nc'
     
     !ե륪ץ
     call netcdf_open(trim(file), ncid)
     
     !˰¾
     if (property(s)%method == 3) then 
        name = "amp"
     elseif (property(s)%method == 4) then 
        name = "antoine"
     end if
     call netcdf_get(ncid, name, rvar)

     !ե륯
     call netcdf_close(ncid)

     !ͤ
     property(s)%svap_cef%A = rvar(1)
     property(s)%svap_cef%B = rvar(2)
     property(s)%svap_cef%C = rvar(3)
     property(s)%svap_cef%D = rvar(4)
     property(s)%svap_cef%E = rvar(5)


     !˰¾иŬѸ³()׻
     cri_low: do i = 1, num
        dt = (temp_high - temp_low) / num 
        temp = temp_high - (i-1) * dt
        
        if (property(s)%method == 3) then 
           call svap_amp(temp, property(s)%svap_cef, svap)
        elseif(property(s)%method == 4) then 
           call svap_antoine(temp, property(s)%svap_cef, svap)
        end if
        
        if (svap <= pres_low) then 
           call entropy(temp, pres_ref, mol, property, entr0=entr0)
           call gibbs_energy(temp, pres_ref, mol, property, mu0=mu0)

           property(s)%svap_cef%low%temp = temp           
           property(s)%svap_cef%low%A = entr0(s)
           property(s)%svap_cef%low%B = mu0(s) + (entr0(s) * temp) / 2.0d0 

           deallocate(entr0, mu0)
           exit cri_low           
        end if
     end do cri_low
     
     
     !˰¾иŬѸ³()׻
     cri_high: do i = 1, num
        dt = (temp_high - temp_low) / num 
        temp = temp_high - (i-1) * dt
        
        if (property(s)%method == 3) then 
           call svap_amp(temp, property(s)%svap_cef, svap)
        elseif(property(s)%method == 4) then 
           call svap_antoine(temp, property(s)%svap_cef, svap)
        end if
       
        if (svap <= pres_high) then 
           call entropy(temp, pres_ref, mol, property, entr0=entr0)
           call entropy(temp - dt, pres_ref, mol, property, entr0=entr0_pre)
           call gibbs_energy(temp, pres_ref, mol, property, mu0=mu0)
           
           entr0_dt = (entr0(s) - entr0_pre(s))/ dt
           property(s)%svap_cef%high%temp = temp                
           property(s)%svap_cef%high%A = entr0_dt
           property(s)%svap_cef%high%B = entr0(s) - entr0_dt * temp
           property(s)%svap_cef%high%C = mu0(s) + entr0(s) * temp &
                & - (entr0_dt * (temp ** 2.0d0)) / 2.0d0 

           deallocate(entr0, entr0_pre, mu0)
           exit cri_high
        end if
     end do cri_high
     
     deallocate(rvar)
     
  end do calc_svap

!!!
!!!夫Ť
!!!
  deallocate(mol)

  
end subroutine def_svap_cef
