!c Description:
!c  ֥ͳͥ륮η׻
!c
!c Current Code Owner: 
!c   sugiyama@gfd-dennou.org
!c
!c Histry: 
!c   Version    Date          Comment
!c   -------    ----------    --------
!c   1.0        2003-02-06    
!c
!c Copyright (C) SUGIYAMA Ko-ichiro, 2003, All rights reserved

module interface_gibbs_energy
  interface

     subroutine gibbs_energy(temp, pres, mol, property, mu0, mu, gibbs)
       use prm_type, only: type_property
       real(8), intent(in)                         :: temp      
       real(8), intent(in)                         :: pres      
       real(8), intent(in)                         :: mol(:)    
       real(8), intent(out), optional, allocatable :: mu0(:)    
       real(8), intent(out), optional, allocatable :: mu(:)     
       real(8), intent(out), optional              :: gibbs     
       type(type_property), intent(in)             :: property(:)
     end subroutine gibbs_energy

  end interface
end module interface_gibbs_energy




subroutine gibbs_energy(temp, pres, mol, property, mu0, mu, gibbs)
  use prm_phys, only: gasr, pres_ref
  use prm_type, only: type_property
  use interface_molfraction
  use interface_cp_spl
  use interface_cp_noblegas
  use interface_svap_amp
  use interface_svap_antoine
  use interface_phase2alpha
  use interface_name_cnd2gas
  use interface_name2num
  use interface_stop
  
  implicit none
  real(8), intent(in)                         :: temp    
  real(8), intent(in)                         :: pres    
  real(8), intent(in)                         :: mol(:)  
  real(8), intent(out), optional, allocatable :: mu0(:)
  real(8), intent(out), optional, allocatable :: mu(:) 
  real(8), intent(out), optional              :: gibbs 
  type(type_property), intent(in)             :: property(:) 
  
  real(8)                        :: mu0_gas       !ɸಽإݥƥ󥷥
  real(8)                        :: cp_int        !Ǯβʬ
  real(8)                        :: cp_dvt_int    !Ǯ/٤βʬ
  real(8)                        :: svap          !˰¾
  real(8), allocatable           :: mu0_tmp(:)    !ɸಽإݥƥ󥷥
  real(8), allocatable           :: mu_tmp(:)     !إݥƥ󥷥
  real(8), allocatable           :: molfr(:)      !
  real(8), allocatable           :: alpha(:)      !
  integer                        :: s
  integer                        :: num
  integer                        :: smax
  character(40)                  :: gasname  


!!!
!!!ѿν
!!!  
  smax = size(property, 1)
  allocate(mu0_tmp(smax), mu_tmp(smax))
  mu0_tmp = 0.0D0
  mu_tmp = 0.0D0
  call phase2alpha(property%phase, alpha)
  call molfraction(mol, property%phase, molfr)

!!!
!!!ɸಽإݥƥ󥷥, إݥƥ󥷥η׻
!!!
  CalcMu0: do s = 1, smax
!!!
!!!ѿν
!!!
     mu0_gas = 0.0D0
     cp_int = 0.0D0
     cp_dvt_int = 0.0D0
     svap = 0.0D0
     
!!!
!!!٥뤬 "NobleGas" ξ
!!!
     if (property(s)%method == 1) then 
        !Ǯ, Ǯ/٤ʬ
        call cp_nobleGas(temp, cp_int=cp_int, cp_dvt_int=cp_dvt_int)        

        !ɸಽإݥƥ󥷥η׻        
        mu0_tmp(s) =  property(s)%ref%enth - temp * property(s)%ref%entr  &
             & + cp_int - temp * cp_dvt_int                     
        
!!!
!!!٥뤬 "Gas" ξ
!!!
     elseif (property(s)%method == 2) then
        !Ǯ, Ǯ/٤ʬ
        call cp_spl(temp, property(s), cp_int=cp_int, cp_dvt_int=cp_dvt_int)
        
        !ɸಽإݥƥ󥷥η׻
        mu0_tmp(s) = property(s)%ref%enth - temp * property(s)%ref%entr &
             & + cp_int - temp * cp_dvt_int
              
!!!
!!!٥뤬˰¾(AMP)ξ
!!!
     elseif (property(s)%method == 3) then 

        if (temp < property(s)%svap_cef%low%temp) then 
           mu0_tmp(s) = - (property(s)%svap_cef%low%A * temp ** 2.0d0)&
                & / (2.0d0 * property(s)%svap_cef%low%temp) &
                & + property(s)%svap_cef%low%B
           
        elseif (temp > property(s)%svap_cef%high%temp) then 
           mu0_tmp(s) = - (property(s)%svap_cef%high%A * (temp ** 2.0d0)) &
                & / 2.0d0 - property(s)%svap_cef%high%B * temp &
                & + property(s)%svap_cef%high%C
          
        else
           !˰¾η׻
           call svap_amp(temp, property(s)%svap_cef, svap)

           !б뵤Υȥԡ
           call name_cnd2gas(property(s)%name, gasname)
           call name2num(gasname, property%name, num)

           mu0_gas = mu0_tmp(num)
           
           !ɸಽإݥƥ󥷥η׻
           mu0_tmp(s)  = mu0_gas + gasr * temp * dlog(svap/pres_ref)

        end if

!!!
!!!٥뤬˰¾(Antoine)ξ
!!!
     elseif (property(s)%method == 4) then 
        
        if (temp < property(s)%svap_cef%low%temp) then 
           mu0_tmp(s) = - (property(s)%svap_cef%low%A * temp ** 2.0d0)&
                & / (2.0d0 * property(s)%svap_cef%low%temp) &
                & + property(s)%svap_cef%low%B
           
        elseif (temp > property(s)%svap_cef%high%temp) then 
           mu0_tmp(s) = - (property(s)%svap_cef%high%A * (temp ** 2.0d0)) &
                & / 2.0d0 - property(s)%svap_cef%high%B * temp &
                & + property(s)%svap_cef%high%C

        else
           !˰¾η׻
           call svap_antoine(temp, property(s)%svap_cef, svap)
           
           !б뵤Υȥԡ
           call name_cnd2gas(property(s)%name, gasname)
           call name2num(gasname, property%name, num)
           mu0_gas = mu0_tmp(num)
           
           !ɸಽإݥƥ󥷥η׻
           mu0_tmp(s)  = mu0_gas + gasr * temp * dlog(svap/pres_ref)
           
        end if


!!!
!!!٥뤬 NH4SH ξ
!!!
     elseif (property(s)%method == 5) then

        !б뵤Υȥԡ
        gasname = 'H2S-g'
        call name2num(gasname, property%name, num)
        mu0_gas = mu0_tmp(num)
        
        !б뵤Υȥԡ
        gasname = 'NH3-g'
        call name2num(gasname, property%name, num)
        mu0_gas = mu0_gas + mu0_tmp(num)        
        
        !ɸಽإݥƥ󥷥η׻
        mu0_tmp(s)  = mu0_gas &
             & + gasr * temp * (61.781d0 - (10834.0d0 / temp)) &
             & - gasr * temp * dlog(1.0d12)


!!!
!!!٥뤬 unknown ʾ
!!!
     else
        call stop(property(s)%method, 'gibbs_energy.f90', 'unkown method')
     end if

  end do CalcMu0


!!!
!!!إݥƥ󥷥η׻
!!!
  mu_tmp = mu0_tmp + gasr * temp * dlog(molfr) &
       & + alpha * gasr * temp * dlog(pres / pres_ref)


!!!
!!!
!!!
  !ɸಽإݥƥ󥷥
  if (present(mu0)) then 
     allocate(mu0(smax))
     mu0 = mu0_tmp
  end if

  !إݥƥ󥷥
  if (present(mu)) then 
     allocate(mu(smax))
     mu = mu_tmp
  end if

  !֥ͳͥ륮
  if (present(gibbs)) gibbs = dot_product(mol, mu_tmp)
  

!!!
!!!
!!!
  deallocate(mu0_tmp, mu_tmp, molfr, alpha)

end subroutine gibbs_energy


